{- markdown2html:

    A program that converts Markdown to HTML using Pandoc
    but handles wikiLinks a little differently, i.e., local
    text files are refered to by their names without extensions.

    Copyright (C) 2009 Paul Liu <paul@thev.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-}

module Main (main) where

import Text.Pandoc hiding (Str)
import Text.Pandoc.Shared (ObfuscationMethod(..))
import System.Environment (getProgName, getArgs)
import System.FilePath (replaceDirectory, addExtension, replaceExtension, 
                        dropExtension, takeExtension)
import System.Directory (getModificationTime)
import System.IO (hPutStrLn, stderr)
import System.Time (toCalendarTime, calendarTimeToString)
import System.Exit
import qualified Data.ByteString.Lazy as L
import Data.ByteString.Lazy.UTF8 (fromString, toString)
import Data.Generics (everywhere, mkT)
import Data.List (isPrefixOf)
  
main = do
  prog <- getProgName
  args <- getArgs
  let templfile = selectOption "--template=" args  
      templopts = map breakit (selectOption "--template-option=" args) 
      diropts   = selectOption "--outputdir=" args
      help      = selectOption "--help" args
      moddir    = if null diropts then id 
                    else flip replaceDirectory (last diropts)
      files     = omitOptions args
  templ <- if null templfile then return defaultTemplate
              else readFile (last templfile) 
  r <- case (null help, null files) of
         (False, _) -> hPutStrLn stderr (usage prog) >> return []
         (_, True)  -> L.getContents >>= L.putStrLn . toHtml templ templopts >> return []
         _          -> mapM (fileToHtml moddir templ templopts) files
  exitWith $ if all id r then ExitSuccess else ExitFailure 1
  where
    selectOption opt = map (tail . dropWhile (/='=')) . filter (isPrefixOf opt) 
    omitOptions = filter (not . isPrefixOf "--") 
    breakit s = let (x, y) = break (=='=') s in (x, if null y then y else tail y)


fileToHtml moddir templ templopts f = do
  let f' = moddir (replaceExtension f "html")
  catch (toHtml' f f' >> return True) 
        (\err -> hPutStrLn stderr (show err) >> return False)
  where
    toHtml' f f' = do
      s <- L.readFile f
      t <- getModificationTime f >>= toCalendarTime 
      let opts = ("date", calendarTimeToString t) : templopts
      L.writeFile f' (toHtml templ opts s)

toHtml templ templopts = 
  fromString . writeHtmlString (writerOption templ templopts)
  . wikiLinksTransform . readMarkdown parserState . toString  

parserState = defaultParserState 
  { stateSanitizeHTML = True
  , stateSmart        = True }

writerOption templ templopts = defaultWriterOptions 
  { writerWrapText         = False
  , writerStandalone       = True
  , writerTemplate         = templ
  , writerVariables        = templopts
  , writerEmailObfuscation = ReferenceObfuscation
  , writerTableOfContents  = True
  }

wikiLinksTransform :: Pandoc -> Pandoc
wikiLinksTransform = everywhere $ mkT convertWikiLinks
  where
    convertWikiLinks x@(Link ref (s, t)) =
      if elem ':' s || not (null (takeExtension s))
        then x
        else Link ref (addExtension s "html", t)
    convertWikiLinks x = x

defaultTemplate = unlines 
  [ "<html><header>"
  , "<title>$if(pagetitle)$$pagetitle$$endif$</title>"
  , "$for(css)$<link rel=\"stylesheet\" href=\"$css$\" type=\"text/css\" />$endfor$"
  , "</header><body><div id=\"wrapper\"><div class=\"generic fiveeighty\">"
  , "$if(title)$<h1 class=\"title\">$title$</h1>$endif$"
  , "$if(toc)$$toc$$endif$"
  , "$body$</div></div></body></html>" ]

usage prog = unlines
  [ "Usage: " ++ prog ++ " [options] [file] ..."
  , "Options: "
  , "  --template=<file>           Pandoc template file."
  , "  --template-option=<option>  Pandoc template option. The option is"
  , "                              specified as var=value, and if more than one"
  , "                              is needed, use another --template-option=..."
  , "  --outputdir=<dir>           Put all outputs into a directory. If it"
  , "                              is not specified, put in the same directory"
  , "                              as the input file."
  , "  --help                      Show this message."
  , "\nBy default, all output HTML files will have the same base name as the"
  , "input files, but with file extension '.html'. If no input file is given,"
  , "it reads from the standard input and writes to the standard output." ]

